use std::result::Result as StdResult;

use wasm_bindgen::prelude::*;

pub mod engine;

/// A type that makes dealing with results easier.
pub type Result<T> = StdResult<T, JsValue>;

#[wasm_bindgen]
extern "C" {
    #[wasm_bindgen(js_namespace = console, js_name = log)]
    pub fn console_log(s: &str);

    #[wasm_bindgen(js_namespace = console, js_name = log)]
    pub fn console_log_js(s: JsValue);

    #[wasm_bindgen(js_name = console, js_name = error)]
    pub fn console_error(s: &str);

    #[wasm_bindgen(js_name = console, js_name = error)]
    pub fn console_error_js(s: JsValue);
}

#[allow(unused)]
macro_rules! log {
    ($($t:tt)*) => (crate::console_log(&format_args!($($t)*).to_string()))
}

#[allow(unused)]
macro_rules! log_js {
    ($t:tt) => {{
        let e: JsValue = $t.into();
        crate::console_log_js(e)
    }};
}

#[allow(unused)]
macro_rules! error {
    ($($t:tt)*) => (crate::console_error(&format_args!($($t)*).to_string()))
}

#[allow(unused)]
macro_rules! error_js {
    ($t:tt) => {{
        let e: JsValue = $t.into();
        crate::console_error_js(e)
    }};
}

#[allow(unused)]
pub(crate) use error;

#[allow(unused)]
pub(crate) use error_js;

#[allow(unused)]
pub(crate) use log;

#[allow(unused)]
pub(crate) use log_js;

pub fn run() -> Result<()> {
    let engine = engine::Engine::new()?;
    engine.start()?;
    Ok(())
}

#[wasm_bindgen(start)]
pub fn start() {
    if let Err(e) = run() {
        error_js!(e);
    }
}
